   1  -- Copyright (C) GIANTS Software GmbH, Confidential, All Rights Reserved.
   2  
   3  ForageWagonAreaEvent = {};
   4  ForageWagonAreaEvent_mt = Class(ForageWagonAreaEvent, Event);
   5  
   6  InitStaticEventClass(ForageWagonAreaEvent, "ForageWagonAreaEvent", EventIds.EVENT_FORAGE_WAGON_AREA);
   7  
   8  function ForageWagonAreaEvent:emptyNew()
   9      local self = Event:new(ForageWagonAreaEvent_mt);
  10      return self;
  11  end;
  12  
  13  function ForageWagonAreaEvent:new(cuttingAreas, fruitType)
  14      local self = ForageWagonAreaEvent:emptyNew()
  15      self.cuttingAreas = cuttingAreas;
  16      self.fruitType = fruitType;
  17      return self;
  18  end;
  19  
  20  function ForageWagonAreaEvent:readStream(streamId, connection)
  21      local fruitType = streamReadUIntN(streamId, FruitUtil.sendNumBits);
  22      local numAreas = streamReadUIntN(streamId, 4);
  23  
  24      local fruitDesc = FruitUtil.fruitIndexToDesc[fruitType]
  25  
  26      local refX = streamReadFloat32(streamId);
  27      local refY = streamReadFloat32(streamId);
  28      local values = Utils.readCompressed2DVectors(streamId, refX, refY, numAreas*3-1, 0.01, true);
  29      for i=1,numAreas do
  30          local vi = i-1;
  31          local x = values[vi*3+1].x;
  32          local z = values[vi*3+1].y;
  33          local x1 = values[vi*3+2].x;
  34          local z1 = values[vi*3+2].y;
  35          local x2 = values[vi*3+3].x;
  36          local z2 = values[vi*3+3].y;
  37  
  38          Utils.updateFruitWindrowArea(fruitType, x, z, x1, z1, x2, z2, 0);
  39          if fruitDesc ~= nil then
  40              for fruitTypeSrc,_ in pairs(fruitDesc.forageWagonConversionSources) do
  41                  Utils.updateFruitWindrowArea(fruitTypeSrc, x, z, x1, z1, x2, z2, 0);
  42              end
  43          end
  44      end;
  45  end;
  46  
  47  function ForageWagonAreaEvent:writeStream(streamId, connection)
  48      streamWriteUIntN(streamId, self.fruitType, FruitUtil.sendNumBits);
  49      local numAreas = table.getn(self.cuttingAreas);
  50      streamWriteUIntN(streamId, numAreas, 4);
  51      local refX, refY;
  52      local values = {};
  53      for i=1, numAreas do
  54          local d = self.cuttingAreas[i];
  55          if i==1 then
  56              refX = d[1];
  57              refY = d[2];
  58              streamWriteFloat32(streamId, d[1]);
  59              streamWriteFloat32(streamId, d[2]);
  60          else
  61              table.insert(values, {x=d[1], y=d[2]});
  62          end;
  63          table.insert(values, {x=d[3], y=d[4]});
  64          table.insert(values, {x=d[5], y=d[6]});
  65      end;
  66      assert(table.getn(values) == numAreas*3 - 1);
  67      Utils.writeCompressed2DVectors(streamId, refX, refY, values, 0.01);
  68  end;
  69  
  70  function ForageWagonAreaEvent:run(connection)
  71      print("Error: Do not run ForageWagonAreaEvent locally");
  72  end;
  73  
  74  function ForageWagonAreaEvent.runLocally(cuttingAreas, fillTypes, fruitTypeFix, fruitType)
  75  
  76      local numAreas = table.getn(cuttingAreas);
  77  
  78      local refX, refY;
  79      local values = {};
  80      for i=1, numAreas do
  81          local d = cuttingAreas[i];
  82          if i==1 then
  83              refX = d[1];
  84              refY = d[2];
  85          else
  86              table.insert(values, {x=d[1], y=d[2]});
  87          end;
  88          table.insert(values, {x=d[3], y=d[4]});
  89          table.insert(values, {x=d[5], y=d[6]});
  90      end;
  91      assert(table.getn(values) == numAreas*3 - 1);
  92  
  93      local values = Utils.simWriteCompressed2DVectors(refX, refY, values, 0.01, true);
  94  
  95      local totalArea = 0;
  96      for i=1, numAreas do
  97          local vi = i-1;
  98          local x = values[vi*3+1].x;
  99          local z = values[vi*3+1].y;
 100          local x1 = values[vi*3+2].x;
 101          local z1 = values[vi*3+2].y;
 102          local x2 = values[vi*3+3].x;
 103          local z2 = values[vi*3+3].y;
 104  
 105          local area = 0
 106          if fruitTypeFix then
 107              area = Utils.updateFruitWindrowArea(fruitType, x, z, x1, z1, x2, z2, 0);
 108              local fruitDesc = FruitUtil.fruitIndexToDesc[fruitType]
 109              if fruitDesc ~= nil then
 110                  for fruitTypeSrc,_ in pairs(fruitDesc.forageWagonConversionSources) do
 111                      area = area + Utils.updateFruitWindrowArea(fruitTypeSrc, x, z, x1, z1, x2, z2, 0);
 112                  end
 113              end
 114          else
 115              for fillType, enabled in pairs(fillTypes) do
 116                  -- Note: we count both grass and dry grass as grass (ignore dry grass in the loop, and pickup dry grass when cutting normal grass)
 117                  --       and we do the same with barley and wheat windrows
 118                  --       This is generalized with the forageWagonConversionTarget and forageWagonConversionSources attributes of the fruitTypeDesc
 119                  fruitType = FruitUtil.fillTypeToFruitType[fillType];
 120                  local fruitDesc;
 121                  if fruitType ~= nil then
 122                      fruitDesc = FruitUtil.fruitIndexToDesc[fruitType];
 123                  end
 124                  if enabled and fruitDesc ~= nil and FruitUtil.fillTypeIsWindrow[fillType] and fruitDesc.forageWagonConversionTarget == nil then
 125                      area = Utils.updateFruitWindrowArea(fruitType, x, z, x1, z1, x2, z2, 0);
 126                      for fruitTypeSrc,_ in pairs(fruitDesc.forageWagonConversionSources) do
 127                          area = area + Utils.updateFruitWindrowArea(fruitTypeSrc, x, z, x1, z1, x2, z2, 0);
 128                      end
 129                      if area > 0 then
 130                          fruitTypeFix = true;
 131                          break;
 132                      end
 133                  end
 134              end
 135          end
 136          if area > 0 then
 137              -- now that we removed the windrow, maybe there is some hidden drygrass to grow (set it to growth state 1 if there is some)
 138              if fruitType == FruitUtil.FRUITTYPE_GRASS then
 139                  Utils.switchFruitTypeArea(FruitUtil.FRUITTYPE_GRASS, FruitUtil.FRUITTYPE_DRYGRASS, x, z, x1, z1, x2, z2, 1);
 140              end
 141              totalArea = totalArea + area;
 142          end
 143      end;
 144      return totalArea, fruitType;
 145  end;